#ifndef __AMSMATH_HPP__
#define __AMSMATH_HPP__

#include <vector>
#include <limits>

namespace ams
{
//#define pi 3.1415926
//OpenGL defines a variable pi - the compiler doesn't like it if I #define a pi also...
static const double pi = 3.14159265358979323846;

//this implementation doesn't work on all complilers?
static const double nan = std::numeric_limits<double>::quiet_NaN();
static const double inf = std::numeric_limits<double>::infinity();

}

#include <amsmath/amsmath_limits.hpp>
#include <amsmath/amsvectorops1.hpp>
#include <amsmath/complex.hpp> //complex number class
#include <amsmath/vect2.hpp>  //a 2-vector and 2-matrix class
#include <amsmath/vect3.hpp>  //a 3-vector and 3-matrix class
#include <amsmath/vect4.hpp>  //a 4-vector and 4-matrix class
#include <amsmath/box.hpp>    //a geometric box class

namespace ams
{
	bool isnan(double V); //needs to test whether x!=x - if true, is NaN. NaN isn't equal to itself.

  double arg(double x, double y);

  //returns azimuth and elevation angles in radians in the standard coordinate system
  //for a given vector (x,y,z)
  int get_azel(double x, double y, double z, double *az, double *el);

  //returns random double from 0 to 1
  double randd();
  //returns random integer from LB to UB
  int randi(int LB,int UB);
  //double dabs(double a)
  //returns double absolute value of double
  double dabs(double a);
  //returns -1. 0, or 1
  double dsgn(double a);

	double min(double a, double b);
	double max(double a, double b);
	long min(long a, long b);
	long max(long a, long b);



}


#endif
