#ifndef __VECT3_HPP__
#define __VECT3_HPP__

namespace ams
{

class vect3
{
public:
  double x,y,z;

  vect3();
  ~vect3();
  vect3& operator=(const vect3& other);
  vect3(const vect3& other);

  vect3 operator+(const vect3& rhs);
  vect3 operator-(const vect3& rhs);
  vect3 operator*(const double& rhs);
  vect3 operator/(const double& rhs);
  friend vect3 operator-(const vect3& rhs);

  double& operator[](int i);

};

class mat3
{
public:

  double a[3][3];

  mat3();
  ~mat3();
  mat3& operator=(const mat3& other);
  mat3(const mat3& other);

  mat3 operator+(const mat3& rhs);
	mat3 operator-(const mat3& rhs);
	mat3 operator*(const double& rhs);
	mat3 operator/(const double& rhs);
	mat3 operator*(const mat3& rhs);
	mat3 operator/(const mat3& rhs);
	friend mat3 operator-(const mat3& rhs);

  //this is cute, but maybe I shouldn't depend on it? It seems ot have some seriously complciated
  //compiler barf associated with constness
	double& operator()(int i, int j);
  const double& operator()(int i, int j) const;


};

vect3 dtov3(double x, double y, double z);
mat3 dtom3(double xx, double xy, double xz, double yx, double yy, double yz, double zx, double zy, double zz);
mat3 transpose(mat3 m1);

mat3 minverse(mat3 m1);
double mdet(mat3 m1);

mat3 mult(mat3 m1, double d);
mat3 mult(double d, mat3 m1);
mat3 mult(mat3 m1, mat3 m2);
vect3 mult(mat3 m1, vect3 v1);
vect3 mult(vect3 v1, mat3 m2);

double vdot(vect3 v1, vect3 v2);
vect3 normalize(vect3 v1);
double vnorm(vect3 v1);
vect3 vcross(vect3 v1, vect3 v2);
vect3 vproj(vect3 v1, vect3 v2);


vect3 randvect3();

//returns azimuth and elevation angles in radians in the standard coordinate system
//for a given vector (x,y,z)
int get_azel(vect3 v, double *az, double *el);

};
#endif //__VECT3_HPP__
