#ifndef __AMSARRAY_HEAD_HPP__
#define __AMSARRAY_HEAD_HPP__

//AMS MDArray: My own array class
//I don't quite like how I have to do things with std::vector. In addition, I want to be able to control some aspects
//of memory allocation directly that std::vector doesn't seem to allow.

#include <stddef.h> //size_t
#include <stdint.h> //intN_t types
#include <vector>

namespace ams
{

  template<typename T> class amsarray
  {
  public:
    T *data;
    long length;    //length of allocated vector
    long reserved;  //length of allocated memory - beyond length is the default object
    double growfactor; //when appending past the length of the array, grow reserved memory by this amount
    bool autoshrink; //shrinks array to fit whenever elements are erased
    //for speed - growfactor set greater than 1
    //for memory - growfactor set to 1.0 or less, autoshrink set to 1;

    amsarray();
    ~amsarray();
    amsarray<T>& operator=(const amsarray<T>& other);
    amsarray(const amsarray<T>& other);


    T& operator[](const long ind);
    const T& operator[](const long ind) const; //perhaps something that will be needed on my vector classes also?
    T& at(const long ind);

    long size();
    long capacity();
    void reserve(const long newsize);
    void resize(const long newsize);
    void shrink_to_fit();
    void clear(); //erases all elements, sets size to 0;
    void setall(const T& elem); //sets everything (from 0 to len) to elem

    bool operator==(const amsarray<T>& other);
    bool operator!=(const amsarray<T>& other);

    //item to insert an element before.
    //0 - prepends. length+1 - appends
    void insert(const T& elem, long loc);
    //void insert(const amsarray<T>& array, size_t loc); //later

    void erase(long loc);
    void erase(long loc1, long loc2);

    void append(const T& elem);
    void prepend(const T& elem);
    void push_back(const T& elem);
    void push_front(const T& elem);

    //find an element in a list
    //returns -1 if it doesn't exist
    long find(const T& elem);

    //finds an element in a sorted list, starting from indstart
    //returns -1 if it doesn't exist
    long find(const T& elem, long indstart);

    T pop_back();
    T pop_front();

  };

  ///////////////////////
  //Function Prototypes//
  ///////////////////////

  //find an element in a sorted list (O(log(n)))
  //returns -1 if the element does not exist
  //requires ordering operators to be defined (< >)
  template<typename T> long find_sorted(ams::amsarray<T> *array, const T& elem);

  //returns where an element T may be inserted
  //such that the ascending order of the list is preserved
  //requires ordering operators to be defined (< >)
  template<typename T> long place_sorted(ams::amsarray<T> *array,const T& elem);

  //inserts an element into the array in such a way that ordering is preserved
  //requires ordering operators to be defined (< >)
  template<typename T> void insert_sorted(ams::amsarray<T> *array,const T& elem);

  template<typename T> void vector_to_amsarray(const std::vector<T> *in, ams::amsarray<T> *out);
  template<typename T> void amsarray_to_vector(const ams::amsarray<T> *in, std::vector<T> *out);

  void printf_array(amsarray<double> *Q, bool newline=0);
  void printf_array(amsarray<long> *Q, bool newline=0);
  void printf_array(amsarray<int> *Q, bool newline=0);

}

#endif
