#ifndef __AMSARRAY_HPP__
#define __AMSARRAY_HPP__

//AMS MDArray: My own array class
//I don't quite like how I have to do things with std::vector. In addition, I want to be able to control some aspects
//of memory allocation directly that std::vector doesn't seem to allow.

#include <stddef.h> //size_t
#include <stdint.h> //intN_t types
#include <vector>

namespace ams
{

////////////////////////////////////////////////////
//Function/Class Declarations in amsarray.head.hpp//
////////////////////////////////////////////////////

////////////////////////
//Function Definitions//
////////////////////////

//find an element in a sorted list (O(log(n)))
//returns -1 if the element does not exist
//requires ordering operators to be defined (< >)
template<typename T> long find_sorted(ams::amsarray<T> *array, const T& elem)
{
  long ret = -1;
  long A = 0;
  long B = array->size();
  long C = (B-A)/2+A;
  bool qf = 0;

  while(qf==0)
  {
    if(elem==array->at(C))
    {
      ret = C;
      qf = 1;
    }
    else if(elem<array->at(C))
    {
      B = C-1;
    }
    else
    {
      A = C+1;
    }
    A = max<long>(A,0); A = min<long>(A,array->size()-1);
    B = max<long>(B,0); B = min<long>(B,array->size()-1);
    C = (B-A)/2+A;
    C = max<long>(C,0); C = min<long>(C,array->size());

    if(qf==0&& ((B<A)||B==A))
    {
      if(array->at(B)==elem)
      {
        ret = B;
      }
      if(array->at(A)==elem)
      {
        ret = A;
      }
      qf = 1;
    }
  }

  return ret;
}

template<typename T> long place_sorted(ams::amsarray<T> *array, const T& elem)
{
  long ret = -1;
  long A = 0;
  long B = array->size();
  long C = (B-A)/2+A;
  bool qf = 0;

  if(array->size()>0)
  {
    while(qf==0)
    {
      //printf("debug1 (A,B,C) = (%ld,%ld,%ld)\n",A,B,C);fflush(stdout);
      C = max<long>(C,0); C = min<long>(C,array->size());
      if(elem==array->at(C))
      {
        ret = C;
        qf = 1;
      }
      else if(elem<array->at(C))
      {
        B = C;
      }
      else
      {
        A = C;
      }
      //printf("debug2 (A,B,C) = (%ld,%ld,%ld)\n",A,B,C);fflush(stdout);
      A = max<long>(A,0); A = min<long>(A,array->size());
      B = max<long>(B,0); B = min<long>(B,array->size());
      C = (B-A)/2+A;
      C = max<long>(C,0); C = min<long>(C,array->size()-1);
      //C = max<long>(C,A); C = min<long>(C,B);
      //printf("debug3 (A,B,C) = (%ld,%ld,%ld)\n",A,B,C);fflush(stdout);
      if(qf==0&& ((B-1)<=A))
      {
        //handle aftermath - I'm flailing here chasing werid bugs - this is UGLY
        C = B;
        C = max<long>(C,0); C = min<long>(C,array->size()-1);
        if(elem<array->at(A))
        {
          ret = A;
        }
        else if(elem>array->at(C))
        {
          C = B+1;
          C = min<long>(C,array->size());
          ret = C;
        }
        else if(elem<array->at(C))
        {
          ret = B;
        }
        else
        {
          ret = C;
        }

        qf = 1;
      }
    }
  }
  else
  {
    ret = 0;
  }


  return ret;
}

template<typename T> void insert_sorted(ams::amsarray<T> *array, const T& elem)
{
  long pos = place_sorted<T>(array,elem);
  array->insert(elem,pos);
  return;
}

template<typename T> void vector_to_amsarray(const std::vector<T> *in, ams::amsarray<T> *out)
{
  long I;
  out->resize(in->size());
  for(I=0;I<out->size();I++)
  {
    out->at(I) = in->at(I);
  }
  return;
}

template<typename T> void vector_to_amsarray(const ams::amsarray<T> *in, std::vector<T> *out)
{
  long I;
  out->resize(in->size());
  for(I=0;I<out->size();I++)
  {
    out->at(I) = in->at(I);
  }
  return;
}

template<typename T> amsarray<T>::amsarray()
{
  data = NULL;
  length = 0;
  reserved = 0;
  growfactor = 1.5;
  autoshrink = 0;
  return;
}

template<typename T> amsarray<T>::~amsarray()
{
  if(data!=NULL)
  {
    delete[] data;
  }
  length = 0;
  reserved = 0;
  growfactor = 0.0;
  autoshrink = 0;
  return;
}

template<typename T> amsarray<T>& amsarray<T>::operator=(const amsarray<T>& other)
{
  long I;
  if(this!=&other)
  {
    delete[] data;
    data = new T[other.reserved];
    length = other.length;
    reserved = other.reserved;
    for(I=0;I<reserved;I++)
    {
      data[I] = other.data[I];
    }
  }
  return *this;
}

template<typename T> amsarray<T>::amsarray(const amsarray<T>& other)
{
  data = NULL;
  length = 0;
  reserved = 0;
  growfactor = 1.5;
  autoshrink = 0;
  if(this!=&other)
  {
    *this = other;
  }
  return;
}

template<typename T> T& amsarray<T>::operator[](const long ind)
{
  return data[ind];
}

template<typename T> const T& amsarray<T>::operator[](const long ind) const
{
  return data[ind];
}

template<typename T> T& amsarray<T>::at(const long ind)
{
  return data[ind];
}

template<typename T> long amsarray<T>::size()
{
  return length;
}

template<typename T> long amsarray<T>::capacity()
{
  return reserved;
}

template<typename T> void amsarray<T>::reserve(const long newsize)
{
  T newvals = T();
  if(newsize>0)
  {
    T* newdata = new T[newsize];
    long I;
    for(I=0;I<newsize&&I<length;I++)
    {
      newdata[I] = data[I];
    }
    for(I=length;I<newsize;I++)
    {
      newdata[I] = newvals; //initializes new data to default values
      //sets numeric types to 0
    }
    // for(I=size;I<newsize;I++)
    // {
    //   //leave populated with default object
    // }
    if(data!=NULL) delete[] data;
    data = newdata;
    reserved = newsize;
    if(reserved<=length) length = reserved;
  }
  else
  {
    if(data!=NULL) delete[] data;
    data = NULL;
    reserved = 0;
    length = 0;
  }
  return;
}

template<typename T> void amsarray<T>::resize(const long newsize)
{
  if(newsize>0)
  {
    reserve(newsize);
    length = newsize;
  }
  return;
}

template<typename T> void amsarray<T>::shrink_to_fit()
{
  reserve(length);
  return;
}

template<typename T> bool amsarray<T>::operator==(const amsarray<T>& other)
{
  bool ret = 1;
  long I;
  for(I=0;I<length;I++)
  {
    if(this->at(I)!=other.at(I))
    {
      ret = 0;
      break;
    }
  }
  return ret;
}

template<typename T> bool amsarray<T>::operator!=(const amsarray<T>& other)
{
  return !(*this == other);
}

template<typename T> void amsarray<T>::insert(const T& elem, long loc)
{
  long newlen;
  long I;
  if(loc>=0 && loc<=length)
  {
    if(reserved<length+1)
    {
      //reserve new memory
      if(growfactor>1.0)
      {
        newlen = (long)(growfactor*(double)(length+1));
        newlen = ams::max<long>(newlen,length+1);
        reserve(newlen);
      }
      else
      {
        reserve(length+1);
      }
    }
    //increment length pointer
    length = length+1;

    // for(I=loc+1;I<length;I++)
    // {
    //   this->at(I) = this->at(I-1);
    // }
    for(I=length-1;I>=(loc+1);I--)
    {
      this->at(I) = this->at(I-1);
    }
    this->at(loc) = elem;
  }
  else
  {
    //I may elect to handle inserting something beyond the bounds of the array later
    //Curretnly insert(T,0) prepends to the array (puts T at [0])
    //insert(T,length+1) puts T at the [old_length] spot (appending to the arrray)
  }

  return;
}

template<typename T> void amsarray<T>::erase(long loc)
{
  long I;
  if(loc>=0 && loc<length)
  {
    //only erase an element if you point at a location between [0,len-1]
    //think about adding code to automatically shrink memory useage here...
    for(I=loc;I<length-1;I++)
    {
      this->at(I)=this->at(I+1);
    }
    length = length - 1;

    if(autoshrink)
    {
      shrink_to_fit();
    }
  }
  return;
}

template<typename T> void amsarray<T>::clear()
{
  this->resize(0);
  return;
}

template<typename T> void amsarray<T>::setall(const T& elem)
{
  long I;
  for(I=0;I<length;I++)
  {
    data[I] = elem;
  }
  return;
}

//this is slow and stupid - make it better later
template<typename T> void amsarray<T>::erase(long loc1, long loc2)
{
  int I,J;
  if(loc1>loc2 || loc1<0 || loc2<0)
  {

  }
  else
  {
    if(loc2>=length-1)
    {
      loc2 = length-1;
    }
    I = loc2-loc1+1;
    for(J=0;J<I;J++)
    {
      erase(J);
    }
  }
  return;
}

template<typename T>  void amsarray<T>::append(const T& elem)
{
  insert(elem,length);
  return;
}

template<typename T>  void amsarray<T>::prepend(const T& elem)
{
  insert(elem,0);
  return;
}

template<typename T>  void amsarray<T>::push_back(const T& elem)
{
  append(elem);
  return;
}

template<typename T>  void amsarray<T>::push_front(const T& elem)
{
  prepend(elem);
  return;
}

template<typename T>  T amsarray<T>::pop_back()
{
  T ret = data[length-1];
  erase(length-1);
  return ret;
}

template<typename T>  T amsarray<T>::pop_front()
{
  T ret = data[0];
  erase(0);
  return ret;
}

template<typename T> long amsarray<T>::find(const T& elem, long indstart)
{
  long ret = -1;
  long I;
  for(I=indstart;I<length;I++)
  {
    if(elem==this->at(I))
    {
      ret = I;
      break;
    }
  }
}

template<typename T> long amsarray<T>::find(const T& elem)
{
  return this->find(elem,0);
}



}//namespace

#endif
