#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <amsutil/amsutil.hpp>
#include <amsmath/amsmath.hpp>
#include <amsgeom/amsgeom.hpp>


namespace ams
{
  box2d::box2d()
  {
    xmin[0] = -inf;
    xmin[1] = -inf;
    xmax[0] = inf;
    xmax[1] = inf;
    return;
  }

  box2d::~box2d()
  {
    xmin[0] = 0;
    xmin[1] = 0;
    xmax[0] = 0;
    xmax[1] = 0;
    return;
  }

  box2d& box2d::operator=(const box2d& other)
  {
    if(this!=&other)
    {
      xmin[0] = other.xmin[0];
      xmin[1] = other.xmin[1];
      xmax[0] = other.xmax[0];
      xmax[1] = other.xmax[1];

    }
    return *this;
  }

  box2d::box2d(const box2d& other)
  {
    if(this!=&other)
    {
      *this = other;
    }
    return;
  }

  void box2d::setlimits(ams::vect2 pt1, ams::vect2 pt2)
  {
    xmin[0] = ams::min<double>(pt1[0],pt2[0]);
    xmin[1] = ams::min<double>(pt1[1],pt2[1]);
    xmax[0] = ams::max<double>(pt1[0],pt2[0]);
    xmax[1] = ams::max<double>(pt1[1],pt2[1]);

    return;
  }

  bool box2d::iswithin(ams::vect2 pt)
  {
    bool ret = 1;
    ret = ret && (pt[0]<xmin[0]||pt[0]>xmax[0]);
    ret = ret && (pt[1]<xmin[1]||pt[1]>xmax[1]);
    return ret;
  }

  double box2d::distance(ams::vect2 pt)
  {
    double ret = 0.0;
    if(pt[0]>xmax[0]) {ret = ret + (pt[0]-xmax[0])*(pt[0]-xmax[0]);}
    if(pt[1]>xmax[1]) {ret = ret + (pt[1]-xmax[1])*(pt[1]-xmax[1]);}
    if(pt[0]<xmin[0]) {ret = ret + (pt[0]-xmin[0])*(pt[0]-xmin[0]);}
    if(pt[1]<xmin[1]) {ret = ret + (pt[1]-xmin[1])*(pt[1]-xmin[1]);}
    return sqrt(ret);
  }

  bool box2d::iswithin(double x, double y)
  {
    ams::vect2 pt = dtov2(x,y);
    return iswithin(pt);
  }

  double box2d::distance(double x, double y)
  {
    ams::vect2 pt = dtov2(x,y);
    return distance(pt);
  }


}; //end namespace
