#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <amsutil/amsutil.hpp>
#include <amsmath/amsmath.hpp>
#include <amsgeom/amsgeom.hpp>


namespace ams
{
  box3d::box3d()
  {
    xmin[0] = -inf;
    xmin[1] = -inf;
    xmin[2] = -inf;
    xmax[0] = inf;
    xmax[1] = inf;
    xmax[2] = inf;
    return;
  }

  box3d::~box3d()
  {
    xmin[0] = 0;
    xmin[1] = 0;
    xmin[2] = 0;
    xmax[0] = 0;
    xmax[1] = 0;
    xmax[2] = 0;
    return;
  }

  box3d& box3d::operator=(const box3d& other)
  {
    if(this!=&other)
    {
      xmin[0] = other.xmin[0];
      xmin[1] = other.xmin[1];
      xmin[2] = other.xmin[2];
      xmax[0] = other.xmax[0];
      xmax[1] = other.xmax[1];
      xmax[2] = other.xmax[2];

    }
    return *this;
  }

  box3d::box3d(const box3d& other)
  {
    if(this!=&other)
    {
      *this = other;
    }
    return;
  }

  void box3d::setlimits(ams::vect3 pt1, ams::vect3 pt2)
  {
    xmin[0] = ams::min<double>(pt1[0],pt2[0]);
    xmin[1] = ams::min<double>(pt1[1],pt2[1]);
    xmin[2] = ams::min<double>(pt1[2],pt2[2]);
    xmax[0] = ams::max<double>(pt1[0],pt2[0]);
    xmax[1] = ams::max<double>(pt1[1],pt2[1]);
    xmax[2] = ams::max<double>(pt1[2],pt2[2]);
    return;
  }

  bool box3d::iswithin(ams::vect3 pt)
  {
    bool ret = 1;
    ret = ret && (pt[0]<xmin[0]||pt[0]>xmax[0]);
    ret = ret && (pt[1]<xmin[1]||pt[1]>xmax[1]);
    ret = ret && (pt[2]<xmin[2]||pt[2]>xmax[2]);
    return ret;
  }

  double box3d::distance(ams::vect3 pt)
  {
    double ret = 0.0;
    if(pt[0]>xmax[0]) {ret = ret + (pt[0]-xmax[0])*(pt[0]-xmax[0]);}
    if(pt[1]>xmax[1]) {ret = ret + (pt[1]-xmax[1])*(pt[1]-xmax[1]);}
    if(pt[2]>xmax[2]) {ret = ret + (pt[2]-xmax[2])*(pt[2]-xmax[2]);}
    if(pt[0]<xmin[0]) {ret = ret + (pt[0]-xmin[0])*(pt[0]-xmin[0]);}
    if(pt[1]<xmin[1]) {ret = ret + (pt[1]-xmin[1])*(pt[1]-xmin[1]);}
    if(pt[2]<xmin[2]) {ret = ret + (pt[2]-xmin[2])*(pt[2]-xmin[2]);}
    return sqrt(ret);
  }

  bool box3d::iswithin(double x, double y, double z)
  {
    ams::vect3 pt = dtov3(x,y,z);
    return iswithin(pt);
  }

  double box3d::distance(double x, double y, double z)
  {
    ams::vect3 pt = dtov3(x,y,z);
    return distance(pt);
  }


}; //end namespace
