#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <amsutil/amsutil.hpp>
#include <amsmath/amsmath.hpp>
#include <amsgeom/amsgeom.hpp>


using namespace std;

namespace ams
{

  tree2d::tree2d()
  {
    nodes.resize(0);
    parentset = NULL;
    return;
  }

  tree2d::~tree2d()
  {
    nodes.resize(0);
    parentset = NULL;
    return;
  }

  tree2d& tree2d::operator=(tree2d& other)
  {
    if(this!=&other)
    {
      this->nodes = other.nodes;
      this->parentset = other.parentset;
    }
    return *this;
  }

  tree2d::tree2d(tree2d& other)
  {
    if(this!=&other)
    {
      *this = other;
    }
    return;
  }

  tree2dnode::tree2dnode()
  {
    parent = -1;
    childleft = -1;
    childright = -1;
    parentset = NULL;
    epsilon = 1.0E-9;
    ind = -1;
    axis = -1;
    box.xmin[0] = -inf;
    box.xmax[0] = inf;
    box.xmin[1] = -inf;
    box.xmax[1] = inf;

    return;
  }

  tree2dnode::~tree2dnode()
  {
    parent = -1;
    childleft = -1;
    childright = -1;
    parentset = NULL;
    epsilon = 0.0;
    ind = -1;
    axis = -1;
    box.xmin[0] = -inf;
    box.xmax[0] = inf;
    box.xmin[1] = -inf;
    box.xmax[1] = inf;

    return;
  }

  tree2dnode& tree2dnode::operator=(tree2dnode& other)
  {
    if(this!=&other)
    {
      parent = other.parent;
      childleft = other.childleft;
      childright = other.childright;
      parentset = other.parentset;
      ind = other.ind;
      axis = other.axis;
      box = other.box;
    }
    return *this;
  }

  void tree2d::setparent(pointset2d* ptr)
  {
    parentset = ptr;
    epsilon = parentset->epsilon;
    return;
  }


  tree2dnode::tree2dnode(tree2dnode& other)
  {
    if(this!=&other)
    {
      *this = other;
    }
    return;
  }

  const tree2dnode& tree2dnode::operator=(const tree2dnode& other)
  {
    if(this!=&other)
    {
      parent = other.parent;
      childleft = other.childleft;
      childright = other.childright;
      parentset = other.parentset;
      ind = other.ind;
      axis = other.axis;
      box = other.box;
    }
    return *this;
  }

  tree2dnode::tree2dnode(const tree2dnode& other)
  {
    if(this!=&other)
    {
      *this = other;
    }
    return;
  }

  ams::vect2 tree2d::getpt(long parentind)
  {
    ams::vect2 ret = dtov2(ams::nan,ams::nan);
    if(parentset!=NULL)
    {
      if(parentind>=0&&parentind<parentset->points.size())
      {
        ret = parentset->points[parentind];
      }
    }
    return ret;
  }

  ams::vect2 tree2d::getpt2(long treeind)
  {
    long parentind;
    ams::vect2 ret = dtov2(ams::nan,ams::nan);
    if(parentset!=NULL)
    {

      if(treeind>=0&&treeind<nodes.size())
      {
        parentind = nodes[treeind].ind;
        ret = getpt(parentind);
      }
    }
    return ret;
  }

  long tree2d::haspoint(ams::vect2 pt)
  {
    long ret = -1;
    if(nodes.size()>0)
    {
      long ptr = 0;
      bool qf = 0;
      int ax = 0;
      while(qf==0)
      {
        ams::vect2 curr_pt = parentset->points[nodes[ptr].ind];
        ax = nodes[ptr].axis;
        if(pt_equal(curr_pt, pt))
        {
          //it's this point
          ret = nodes[ptr].ind;
          qf = 1;
        }
        else
        {
          //switch to checking one of the children
          if(pt[ax]<=curr_pt[ax])
          {
            //check child-left
            if(nodes[ptr].childleft<0)
            {
              //there is no child-left - stop searching
              qf = 1;
              ret = -1;
            }
            else
            {
              ptr = nodes[ptr].childleft; //set pointer to childleft
            }
          }
          else
          {
            //check child right
            if(nodes[ptr].childright<0)
            {
              //there is no child-right - stop searching
              qf = 1;
              ret = -1;
            }
            else
            {
              ptr = nodes[ptr].childright; //set pointer to childright
            }
          }
        }

      }
    }
    else
    {
      ret = -1; //we currently don't contain this point
    }

  }

  bool tree2d::pt_equal(ams::vect2 pt1, ams::vect2 pt2)
  {
    bool ret = 0;
    if(ams::dabs(pt1[0]-pt2[0])<epsilon && ams::dabs(pt1[1]-pt2[1])<epsilon)
    {
      ret = 1;
    }
    return ret;
  }

  long tree2d::addpoint(ams::vect2 pt)
  {
    long ret = -1;
    long q = haspoint(pt);

    if(q<0)
    {
      
    }
    else
    {
      ret = q;
    }

    return ret;
  }


};
