#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <vector>
#include <string>

#include <amsmath/amsmath.hpp>


using namespace std;

namespace ams
{
namespace cmp
{

complex::complex()
{
  //initcode
  r = 0.0;
  i = 0.0;
  return;
}

complex::~complex()
{
  //cleanup code
  r = 0.0;
  i = 0.0;
  return;
}

const complex& complex::operator=(const complex &other)
{
  if(this!=&other)
  {
    r = other.r;
    i = other.i;
    //copy code
  }
  return *this;
}

const complex& complex::operator=(const double &other)
{
    r = other;
    i = 0.0;
  return *this;
}

complex::complex(const complex &other)
{
  r = 0.0;
  i = 0.0;
  if(this!=&other)
  {
    *this = other;
  }
  return;
}

complex::complex(double _r, double _i)
{
  i = _i;
  r = _r;
  return;
}

complex::complex(double _r)
{
  r = _r;
  i = 0.0;
  return;
}

complex complex::operator+(const complex &z)
{
  complex cp;
  cp.r = r+z.r;
  cp.i = i+z.i;
  return cp;
}

 complex  complex::operator-(const complex &z)
{
  complex cp;
  cp.r = r-z.r;
  cp.i = i-z.i;
  return cp;
}

 complex complex::operator*(const complex &z)
{
  complex cp;
	cp.r = r * z.r - i * z.i;
	cp.i = r * z.i + i * z.r;
	return cp;
}

 complex complex::operator/(const complex &z)
{
	complex cp;
	cp.r = (r*z.r+i*z.i)/(z.r*z.r+z.i*z.i);
	cp.i = (i*z.r-r*z.i)/(z.r*z.r+z.i*z.i);
	return cp;
}

 complex complex::operator+(const double &z)
{
	complex cp;
	cp.r = r + z;
	cp.i = i + 0.0;
	return cp;
}

 complex complex::operator-(const double &z)
{
	complex cp;
	cp.r = r - z;
	cp.i = i - 0.0;
	return cp;
}

 complex complex::operator*(const double &z)
{
	complex cp;
	cp.r = r * z;
	cp.i = i * z;
	return cp;
}

 complex complex::operator/(const double &z)
{
	complex cp;
	cp.r = r/z;
	cp.i = i/z;
	return cp;
}

complex operator-(const complex &z)
{
	complex ret;
	ret.r = -z.r;
	ret.i = -z.i;
	return ret;
}

double complex::arg()
{
	double ret = ams::arg(this->r,this->i);
	return ret;
}

double complex::mag()
{
	return ::sqrt(r*r+i*i);
}

bool complex::iszero()
{
	bool ret = 0;
	if(r==0.0 && i == 0.0)
  {
    ret = 1;
  }
	return ret;
}

bool complex::isreal()
{
  bool ret = 0;
	// bool ret = 0;
	// if(re>DSMALL||re<-DSMALL)
	// {
	// 	ret = 0;
	// }
	// else
	// {
	// 	ret = 1;
	// }
  if(i==0.0)
  {
    ret = 1;
  }

	return ret;
}

bool complex::isimag()
{
  bool ret = 0;
  if(r==0.0)
  {
    ret = 1;
  }
  return ret;
}

const complex complex::conj()
{
  complex cp;
  cp.r = r;
  cp.i = -i;
  return cp;
}

double real(complex z)
{
  return z.r;
}

double imag(complex z)
{
  return z.i;
}

complex conj(complex z)
{
  complex cp;
  cp.r = z.r;
  cp.i = -z.i;
  return cp;
}

complex exp(complex z)
{
  complex ret;
  ret.r = ::exp(z.r)*::cos(z.i);
  ret.i = ::exp(z.r)*::sin(z.i);
  return ret;
}


complex log(complex z)
{
  complex ret;
  ret.r = ::log(::sqrt(z.r*z.r+z.i*z.i));
  ret.i = ams::arg(z.r,z.i);
  return ret;
}

complex cos(complex z)
{
  complex ret = dtocomp(0.0,0.0);
  complex div = dtocomp(2.0,0.0);
  complex im1 = dtocomp(0.0,1.0);
  ret = (exp(im1*z)+exp(im1*(-z)))/div;
  return ret;
}

complex sin(complex z)
{
  complex ret = dtocomp(0.0,0.0);
  complex div = dtocomp(0.0,2.0);
  complex im1 = dtocomp(0.0,1.0);
  ret = (exp(im1*z)-exp(im1*(-z)))/div;
  return ret;
}

complex cosh(complex z)
{
  complex ret = dtocomp(0.0,0.0);
  complex div = dtocomp(2.0,0.0);

  ret = (exp(z)+exp((-z)))/div;
  return ret;
}

complex sinh(complex z)
{
  complex ret = dtocomp(0.0,0.0);
  complex div = dtocomp(2.0,0.0);

  ret = (exp(z)-exp(-z))/div;
  return ret;
}

complex tan(complex z)
{
	return sin(z)/cos(z);
}

complex tanh(complex z)
{
	return sinh(z)/cosh(z);
}

complex pow(complex z1, complex z2)
{
	complex ret;
	if(z1.mag()>DSMALL)
		ret= exp(z2*log(z1));
	else
		ret = 1.0;
	return ret;
}

complex dtocomp(double _r, double _i)
{
	complex ret;
	ret.r = _r;
	ret.i = _i;
	return ret;
}

double abs(complex z)
{
  return z.mag();
}

double arg(ams::cmp::complex z)
{
  return ams::arg(z.r,z.i);
}

}; //end namespace cmp
}; //end namespace ams
