#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <vector>
#include <string>

#include <amsmath/amsmath.hpp>


using namespace std;

namespace ams
{

vect2::vect2()
{
  x = 0.0; y = 0.0;
  return;
}

vect2::~vect2()
{
  x = 0.0; y = 0.0;
  return;
}

// const vect2& vect2::operator=(const vect2 &other)
// {
//   if(this!=&other)
//   {
//     x = other.x;
//     y = other.y;
//   }
//   return *this;
// }

vect2& vect2::operator=(const vect2 &other)
{
  if(this!=&other)
  {
    x = other.x;
    y = other.y;
  }
  return *this;
}

vect2::vect2(const vect2 &other)
{
  x = 0.0; y = 0.0;
  if(this!=&other)
  {
    *this = other;
  }
  return;
}

mat2::mat2()
{
  a[0][0] = 0.0;
  a[0][1] = 0.0;
  a[1][0] = 0.0;
  a[1][1] = 0.0;
  return;
}

mat2::~mat2()
{
  a[0][0] = 0.0;
  a[0][1] = 0.0;
  a[1][0] = 0.0;
  a[1][1] = 0.0;
  return;
}

mat2& mat2::operator=(const mat2& other)
{
  if(this!=&other)
  {
    a[0][0] = other.a[0][0];
    a[0][1] = other.a[0][1];
    a[1][0] = other.a[1][0];
    a[1][1] = other.a[1][1];
  }
  return *this;
}

mat2::mat2(const mat2& other)
{
  a[0][0] = 0.0;
  a[0][1] = 0.0;
  a[1][0] = 0.0;
  a[1][1] = 0.0;
  if(this!=&other)
  {
    *this = other;
  }
  return;
}

vect2 vect2::operator+(const vect2& rhs)
{
  vect2 ret;
  ret.x = this->x+rhs.x;
  ret.y = this->y+rhs.y;
  return ret;
}

vect2 vect2::operator-(const vect2& rhs)
{
  vect2 ret;
  ret.x = this->x-rhs.x;
  ret.y = this->y-rhs.y;
  return ret;
}

vect2 vect2::operator*(const double rhs)
{
  vect2 ret;
  ret.x = this->x*rhs;
  ret.y = this->y*rhs;
  return ret;
}

vect2 vect2::operator/(const double rhs)
{
  vect2 ret;
  ret.x = this->x/rhs;
  ret.y = this->y/rhs;
  return ret;
}

vect2 operator-(const vect2& rhs)
{
  vect2 ret;
  ret.x = -rhs.x;
  ret.y = -rhs.y;
  return ret;
}

double& vect2::operator[](int i)
{
  if(i==0)
  {
    return x;
  }
  else if(i==1)
  {
    return y;
  }
  else
  {
    printf("Error: vect2::operator[]: %d index is out of bounds [0-1]!\n",i);
    exit(0);
  }
}

mat2 mat2::operator+(const mat2& rhs)
{
  mat2 ret;
  ret.a[0][0] = this->a[0][0]+rhs.a[0][0];
  ret.a[0][1] = this->a[0][1]+rhs.a[0][1];
  ret.a[1][0] = this->a[1][0]+rhs.a[1][0];
  ret.a[1][1] = this->a[1][1]+rhs.a[1][1];

  return ret;
}

mat2 mat2::operator-(const mat2& rhs)
{
  mat2 ret;
  ret.a[0][0] = this->a[0][0]+rhs.a[0][0];
  ret.a[0][1] = this->a[0][1]+rhs.a[0][1];
  ret.a[1][0] = this->a[1][0]+rhs.a[1][0];
  ret.a[1][1] = this->a[1][1]+rhs.a[1][1];

  return ret;
}

mat2 mat2::operator*(const mat2& rhs)
{
  mat2 ret;
  int I,J,K;
  for(I=0;I<2;I++)
  {
    for(J=0;J<2;J++)
    {
      for(K=0;K<2;K++)
      {
        ret(I,J) = ret(I,J) + a[I][K] * rhs(K,J);
      }
    }
  }

  return ret;
}

mat2 mat2::operator/(const mat2& rhs)
{
  mat2 ret;
  mat2 q = minverse(rhs);
  ret = (*this)*q;
  return ret;
}

mat2 mat2::operator*(const double& rhs)
{
  mat2 ret;
  int I,J,K;
  ret(0,0) = a[0][0]*rhs;
  ret(0,1) = a[0][1]*rhs;
  ret(1,0) = a[1][0]*rhs;
  ret(1,1) = a[1][1]*rhs;

  return ret;
}

mat2 mat2::operator/(const double& rhs)
{
  mat2 ret;
  int I,J,K;
  ret(0,0) = a[0][0]/rhs;
  ret(0,1) = a[0][1]/rhs;
  ret(1,0) = a[1][0]/rhs;
  ret(1,1) = a[1][1]/rhs;

  return ret;
}

double& mat2::operator()(int I, int J)
{
  return a[I][J];
}

const double& mat2::operator()(int I, int J) const
{
  return a[I][J];
}

mat2 operator-(const mat2& rhs)
{
  mat2 ret;
  ret(0,0) = -rhs(0,0);
  ret(0,1) = -rhs(0,1);
  ret(1,0) = -rhs(1,0);
  ret(1,1) = -rhs(1,1);

  return ret;
}

double mdet(mat2 m1)
{
  double ret = m1(0,0)*m1(1,1)-m1(1,0)*m1(0,1);
  return ret;
}

mat2 minverse(mat2 m1)
{
  mat2 ret = dtom2(0.0,0.0,0.0,0.0);
	double dt = mdet(m1);
	if(dt>DSMALL||dt<-DSMALL)
	{
		ret.a[0][0] =  (m1.a[1][1])/dt;
		ret.a[0][1] =  -(m1.a[0][1])/dt;
		ret.a[1][0] =  -(m1.a[1][0])/dt;
		ret.a[1][1] =  (m1.a[0][0])/dt;
	}
	return ret;
}

mat2 dtom2(double xx, double xy, double yx, double yy)
{
	mat2 ret;
	ret.a[0][0] = xx;
	ret.a[0][1] = xy;
	ret.a[1][0] = yx;
	ret.a[1][1] = yy;
	return ret;
}

mat2 vtom2(vect2 v1, vect2 v2)
{
	return dtom2(v1.x,v1.y,v2.x,v2.y);
}

mat2 mult(mat2 m1, double d)
{
  mat2 ret;
  ret(0,0) = m1(0,0)*d;
  ret(0,1) = m1(0,1)*d;
  ret(1,0) = m1(1,0)*d;
  ret(1,1) = m1(1,1)*d;
  return ret;
}

mat2 mult(double d, mat2 m1)
{
  mat2 ret;
  ret(0,0) = m1(0,0)*d;
  ret(0,1) = m1(0,1)*d;
  ret(1,0) = m1(1,0)*d;
  ret(1,1) = m1(1,1)*d;
  return ret;
}

mat2 mult(mat2 m1, mat2 m2)
{
  return m1*m2;
}
vect2 mult(mat2 m1, vect2 v1)
{
  vect2 ret;
  ret[0] = m1(0,0)*v1[0]+m1(0,1)*v1[1];
  ret[1] = m1(1,0)*v1[0]+m1(1,1)*v1[1];
  return ret;
}

vect2 mult(vect2 v1, mat2 m1)
{
  vect2 ret;
  ret[0] = m1(0,0)*v1[0]+m1(1,0)*v1[1];
  ret[1] = m1(0,1)*v1[0]+m1(1,1)*v1[1];
  return ret;
}

double vdot(vect2 v1, vect2 v2)
{
  return v1[0]*v2[0]+v1[1]*v2[1];
}

double vnorm(vect2 v1)
{
  return ::sqrt(v1[0]*v1[0]+v1[1]*v1[1]);
}

vect2 normalize(vect2 v2)
{
  vect2 ret = v2/vnorm(v2);
  return ret;
}

double vcross(vect2 v1, vect2 v2)
{
  return v1[0]*v2[1]-v1[1]*v2[0];
}

vect2 vproj(vect2 v1, vect2 v2)
{
  vect2 ret;
  vect2 v22;
  double dt;

  v22 = normalize(v2);
  dt = vdot(v1,v22);
  ret = v22*dt;
  return ret;
}

vect2 dtov2(double x, double y)
{
  vect2 ret;
  ret.x = x; ret.y = y;
  return ret;
}

mat2 transpose(mat2 m1)
{
  mat2 m2;

  m2(0,0) = m1(0,0);
  m2(0,1) = m1(1,0);
  m2(1,0) = m1(0,1);
  m2(1,1) = m1(1,1);

  return m2;
}

vect2 randvect2()
{
	//returns random vector
	//this vector is circularly symmetric in terms of distribution
	vect2 ret;
	double r;
	double theta;
	r = randd();
	theta = 2.0*pi*randd();
	ret.x = r*::cos(theta);
	ret.y = r*::sin(theta);
	return ret;
}

double arg(vect2 v)
{
  return ams::arg(v.x,v.y);
}

double abs(vect2 v)
{
  return vnorm(v);
}

};
