#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <vector>
#include <string>

#include <amsmath/amsmath.hpp>


using namespace std;

namespace ams
{

vect3::vect3()
{
  x = 0.0;
  y = 0.0;
  z = 0.0;
  return;
}

vect3::~vect3()
{
  x = 0.0;
  y = 0.0;
  z = 0.0;
  return;
}

vect3& vect3::operator=(const vect3 &other)
{
  if(this!=&other)
  {
    x = other.x;
    y = other.y;
    z = other.z;
  }
  return *this;
}

vect3::vect3(const vect3 &other)
{
  x = 0.0;
  y = 0.0;
  z = 0.0;
  if(this!=&other)
  {
    *this = other;
  }
  return;
}

double& vect3::operator[](int i)
{
  if(i==0)
  {
    return x;
  }
  else if(i==1)
  {
    return y;
  }
  else if(i==2)
  {
    return z;
  }
  else
  {
    printf("vect3::operator[] %d is out of bounds!\n",i);
    exit(0);
  }
}

mat3::mat3()
{
  int I,J;
  for(I=0;I<3;I++)
  {
    for(J=0;J<3;J++)
    {
      a[I][J] = 0.0;
    }
  }

  return;
}

mat3::~mat3()
{
  int I,J;
  for(I=0;I<3;I++)
  {
    for(J=0;J<3;J++)
    {
      a[I][J] = 0.0;
    }
  }
  
  return;
}

mat3& mat3::operator=(const mat3& other)
{
  int I,J;
  if(this!=&other)
  {
    for(I=0;I<3;I++)
    {
      for(J=0;J<3;J++)
      {
        a[I][J] = other.a[I][J];
      }
    }
  }
  return *this;
}

mat3::mat3(const mat3& other)
{
  int I,J;
  for(I=0;I<3;I++)
  {
    for(J=0;J<3;J++)
    {
      a[I][J] = 0.0;
    }
  }
  if(this!=&other)
  {
    *this = other;
  }
  return;
}

double& mat3::operator()(int i, int j)
{
  return a[i][j];
}
const double& mat3::operator()(int i, int j) const
{
  return a[i][j];
}

vect3 vect3::operator+(const vect3& rhs)
{
  vect3 ret;
  ret.x = x + rhs.x;
  ret.y = y + rhs.y;
  ret.z = z + rhs.z;
  return ret;
}

vect3 vect3::operator-(const vect3& rhs)
{
  vect3 ret;
  ret.x = x - rhs.x;
  ret.y = y - rhs.y;
  ret.z = z - rhs.z;
  return ret;
}

vect3 vect3::operator*(const double& rhs)
{
  vect3 ret;
  ret.x = x * rhs;
  ret.y = y * rhs;
  ret.z = z * rhs;
  return ret;
}

vect3 vect3::operator/(const double& rhs)
{
  vect3 ret;
  ret.x = x / rhs;
  ret.y = y / rhs;
  ret.z = z / rhs;
  return ret;
}

vect3 operator-(const vect3& rhs)
{
  vect3 ret;
  ret.x = -rhs.x;
  ret.y = -rhs.y;
  ret.z = -rhs.z;
  return ret;
}

mat3 mat3::operator+(const mat3& rhs)
{
  mat3 ret;
  int I,J;
  for(I=0;I<3;I++)
  {
    for(J=0;J<3;J++)
    {
      ret(I,J) = a[I][J]+rhs(I,J);
    }
  }
  return ret;
}
mat3 mat3::operator-(const mat3& rhs)
{
  mat3 ret;
  int I,J;
  for(I=0;I<3;I++)
  {
    for(J=0;J<3;J++)
    {
      ret(I,J) = a[I][J]-rhs(I,J);
    }
  }
  return ret;
}
mat3 mat3::operator*(const double& rhs)
{
  mat3 ret;
  int I,J;
  for(I=0;I<3;I++)
  {
    for(J=0;J<3;J++)
    {
      ret(I,J) = a[I][J]*rhs;
    }
  }
  return ret;
}
mat3 mat3::operator/(const double& rhs)
{
  mat3 ret;
  int I,J;
  for(I=0;I<3;I++)
  {
    for(J=0;J<3;J++)
    {
      ret(I,J) = a[I][J]/rhs;
    }
  }
  return ret;
}
mat3 mat3::operator*(const mat3& rhs)
{
  mat3 ret;
  int i,j,k;
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      for(k=0;k<3;k++)
      {
        ret(i,j) = ret(i,j) + a[i][k]*rhs(k,j);
      }
    }
  }
  return ret;
}

mat3 mat3::operator/(const mat3& rhs)
{
  mat3 ret;
  mat3 q = minverse(rhs);
  ret = (*this)*q;
  return ret;
}

mat3 operator-(const mat3& rhs)
{
  mat3 ret;
  int i,j;
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      ret(i,j) = -rhs(i,j);
    }
  }
  return ret;
}

double mdet(mat3 m1)
{
  double ret = 0.0;
  ret = ret + m1.a[0][0]*m1.a[1][1]*m1.a[2][2];
  ret = ret + m1.a[0][1]*m1.a[1][2]*m1.a[2][0];
  ret = ret + m1.a[0][2]*m1.a[1][0]*m1.a[2][1];
  ret = ret - m1.a[0][0]*m1.a[1][2]*m1.a[2][1];
  ret = ret - m1.a[0][1]*m1.a[1][0]*m1.a[2][2];
  ret = ret - m1.a[0][2]*m1.a[1][1]*m1.a[2][0];

  return ret;
}

mat3 minverse(mat3 m1)
{
	mat3 ret = dtom3(0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0);
	double dt = mdet(m1);
	if(dt>DSMALL||dt<-DSMALL)
	{
		ret.a[0][0] =  (m1.a[1][1]*m1.a[2][2]-m1.a[1][2]*m1.a[2][1])/dt;
		ret.a[0][1] = -(m1.a[1][0]*m1.a[2][2]-m1.a[1][2]*m1.a[2][0])/dt;
		ret.a[0][2] =  (m1.a[1][0]*m1.a[2][1]-m1.a[1][1]*m1.a[2][0])/dt;
		ret.a[1][0] = -(m1.a[0][1]*m1.a[2][2]-m1.a[0][2]*m1.a[2][1])/dt;
		ret.a[1][1] =  (m1.a[0][0]*m1.a[2][2]-m1.a[0][2]*m1.a[2][0])/dt;
		ret.a[1][2] = -(m1.a[0][0]*m1.a[2][1]-m1.a[0][1]*m1.a[2][0])/dt;
		ret.a[2][0] =  (m1.a[0][1]*m1.a[1][2]-m1.a[0][2]*m1.a[1][1])/dt;
		ret.a[2][1] = -(m1.a[0][0]*m1.a[1][2]-m1.a[0][2]*m1.a[1][0])/dt;
		ret.a[2][2] =  (m1.a[0][0]*m1.a[1][1]-m1.a[0][1]*m1.a[1][0])/dt;

		ret = transpose(ret);
	}
	return ret;
}

mat3 mult(mat3 m1, double d)
{
  return m1*d;
}
mat3 mult(double d, mat3 m1)
{
  return m1*d;
}
mat3 mult(mat3 m1, mat3 m2)
{
  return m1*m2;
}
vect3 mult(mat3 m1, vect3 v1)
{
  vect3 ret;
  int i,j;
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      ret[i] = m1(i,j)*v1[j];
    }
  }
  return ret;
}
vect3 mult(vect3 v1, mat3 m1)
{
  vect3 ret;
  int i,j;
  for(i=0;i<3;i++)
  {
    for(j=0;j<3;j++)
    {
      ret[i] = m1(j,i)*v1[j];
    }
  }
  return ret;
}

double vdot(vect3 v1, vect3 v2)
{
  return v1[0]*v2[0]+v1[1]*v2[1]+v1[2]*v2[2];
}

vect3 normalize(vect3 v1)
{
  vect3 ret = v1/vnorm(v1);
  return ret;
}

double vnorm(vect3 v1)
{
  return ::sqrt(v1[0]*v1[0]+v1[1]*v1[1]+v1[2]*v1[2]);
}

vect3 vcross(vect3 v1, vect3 v2)
{
	vect3 ret;
	ret.x = v1.y*v2.z - v1.z*v2.y;
	ret.y = v1.z*v2.x - v1.x*v2.z;
	ret.z = v1.x*v2.y - v1.y*v2.x;
	return ret;
}

vect3 vproj(vect3 v1, vect3 v2);

vect3 dtov3(double x, double y, double z)
{
	vect3 ret;
	ret.x = x;
	ret.y = y;
	ret.z = z;
	return ret;
}

mat3 dtom3(double xx, double xy, double xz, double yx, double yy, double yz, double zx, double zy, double zz)
{
	mat3 ret;
	ret.a[0][0] = xx;
	ret.a[0][1] = xy;
	ret.a[0][2] = xz;
	ret.a[1][0] = yx;
	ret.a[1][1] = yy;
	ret.a[1][2] = yz;
	ret.a[2][0] = zx;
	ret.a[2][1] = zy;
	ret.a[2][2] = zz;
	return ret;
}

mat3 transpose(mat3 m1)
{
	mat3 ret;
	int i,j;
	for(i=0;i<3;i++)
	{
		for(j=0;j<3;j++)
		{
			ret.a[j][i] = m1.a[i][j];
		}
	}
	return ret;
}

vect3 randvect3()
{
	//returns random vector
	//this vector is not spherically symmetric in terms of distribution!
	vect3 ret;
	ret.x = 2.0*randd()-1.0;
	ret.y = 2.0*randd()-1.0;
	ret.z = 2.0*randd()-1.0;
	return ret;
}

//returns azimuth and elevation angles in radians in the standard coordinate system
//for a given vector (x,y,z)
int get_azel(vect3 v, double *az, double *el)
{
	int ret = get_azel(v.x,v.y,v.z,az,el);
	return ret;
}

};
