#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <vector>
#include <string>

#include <amsmath/amsmath.hpp>


using namespace std;

namespace ams
{

vect4::vect4()
{
  a[0] = 0.0;
  a[1] = 0.0;
  a[2] = 0.0;
  a[3] = 0.0;

  return;
}

vect4::~vect4()
{
  //cleanup code
  a[0] = 0.0;
  a[1] = 0.0;
  a[2] = 0.0;
  a[3] = 0.0;
  return;
}

vect4& vect4::operator=(const vect4 &other)
{
  if(this!=&other)
  {
    a[0] = other.a[0];
    a[1] = other.a[1];
    a[2] = other.a[2];
    a[3] = other.a[3];
  }
  return *this;
}

vect4::vect4(const vect4 &other)
{
  a[0] = 0.0;
  a[1] = 0.0;
  a[2] = 0.0;
  a[3] = 0.0;
  if(this!=&other)
  {
    *this = other;
  }
  return;
}

mat4::mat4()
{
  int i;
  for(i=0;i<16;i++)
  {
      a[i] = 0.0;
  }
  zero = 0.0;
  return;
}

mat4::~mat4()
{
  int i;
  for(i=0;i<16;i++)
  {
      a[i] = 0.0;
  }
  zero = 0.0;
  return;
}

double& mat4::at(int I, int J)
{
  // http://www.opengl-tutorial.org/assets/faq_quaternions/index.html#I1
  // http://stackoverflow.com/questions/4360918/correct-opengl-matrix-format
  // states order for OpenGL matrices. I'm going to use this convention so I
  // can blit the internal data array to opengl.

  //             This Document                  OpenGL
  //
  //            | 0  1  2  3  |            | 0  4  8  12 |
  //            |             |            |             |
  //            | 4  5  6  7  |            | 1  5  9  13 |
  //        M = |             |        M = |             |
  //            | 8  9  10 11 |            | 2  6  10 14 |
  //            |             |            |             |
  //            | 12 13 14 15 |            | 3  7  11 15 |

  if(I>=0&&I<4&&J>=0&&J<4)
  {
    //openGL uses row major order?
    return a[I+4*J];
  }
  else
  {
    return zero;
  }
}

const double& mat4::at(int I, int J) const
{
  // http://www.opengl-tutorial.org/assets/faq_quaternions/index.html#I1
  // http://stackoverflow.com/questions/4360918/correct-opengl-matrix-format
  // states order for OpenGL matrices. I'm going to use this convention so I
  // can blit the internal data array to opengl.

  //             This Document                  OpenGL
  //
  //            | 0  1  2  3  |            | 0  4  8  12 |
  //            |             |            |             |
  //            | 4  5  6  7  |            | 1  5  9  13 |
  //        M = |             |        M = |             |
  //            | 8  9  10 11 |            | 2  6  10 14 |
  //            |             |            |             |
  //            | 12 13 14 15 |            | 3  7  11 15 |

  if(I>=0&&I<4&&J>=0&&J<4)
  {
    //openGL uses row major order?
    return a[I+4*J];
  }
  else
  {
    return zero;
  }
}

mat4& mat4::operator=(const mat4& other)
{
  if(this!=&other)
  {
    int i,j;
    for(i=0;i<4;i++)
    {
      for(j=0;j<4;j++)
      {
        this->at(i,j)= other.at(i,j);
      }
    }
    zero = 0.0;
  }
  return *this;
}

mat4::mat4(const mat4& other)
{
  int i,j;
  for(i=0;i<16;i++)
  {
      a[i] = 0.0;
  }
  zero = 0.0;
  if(this!=&other)
  {
    *this = other;
  }
  return;
}

double& vect4::operator[](int I)
{
  return a[I];
}

double& vect4::at(int I)
{
  return a[I];
}

double& mat4::operator()(int I, int J)
{
  // http://www.opengl-tutorial.org/assets/faq_quaternions/index.html#I1
  // http://stackoverflow.com/questions/4360918/correct-opengl-matrix-format
  // states order for OpenGL matrices. I'm going to use this convention so I
  // can blit the internal data array to opengl.

  //             This Document                  OpenGL
  //
  //            | 0  1  2  3  |            | 0  4  8  12 |
  //            |             |            |             |
  //            | 4  5  6  7  |            | 1  5  9  13 |
  //        M = |             |        M = |             |
  //            | 8  9  10 11 |            | 2  6  10 14 |
  //            |             |            |             |
  //            | 12 13 14 15 |            | 3  7  11 15 |

  if(I>=0&&I<4&&J>=0&&J<4)
  {
    //openGL uses row major order?
    return a[I+4*J];
  }
  else
  {
    return zero;
  }
}

const double& mat4::operator()(int I, int J) const
{
  // http://www.opengl-tutorial.org/assets/faq_quaternions/index.html#I1
  // http://stackoverflow.com/questions/4360918/correct-opengl-matrix-format
  // states order for OpenGL matrices. I'm going to use this convention so I
  // can blit the internal data array to opengl.

  //             This Document                  OpenGL
  //
  //            | 0  1  2  3  |            | 0  4  8  12 |
  //            |             |            |             |
  //            | 4  5  6  7  |            | 1  5  9  13 |
  //        M = |             |        M = |             |
  //            | 8  9  10 11 |            | 2  6  10 14 |
  //            |             |            |             |
  //            | 12 13 14 15 |            | 3  7  11 15 |

  if(I>=0&&I<4&&J>=0&&J<4)
  {
    //openGL uses row major order?
    return a[I+4*J];
  }
  else
  {
    return zero;
  }
}

vect4 vect4::operator+(const vect4& rhs)
{
  vect4 ret;
  ret.a[0] = this->a[0] + rhs.a[0];
  ret.a[1] = this->a[1] + rhs.a[1];
  ret.a[2] = this->a[2] + rhs.a[2];
  ret.a[3] = this->a[3] + rhs.a[3];
  return ret;
}

vect4 vect4::operator-(const vect4& rhs)
{
  vect4 ret;
  ret.a[0] = this->a[0] - rhs.a[0];
  ret.a[1] = this->a[1] - rhs.a[1];
  ret.a[2] = this->a[2] - rhs.a[2];
  ret.a[3] = this->a[3] - rhs.a[3];
  return ret;
}

vect4 vect4::operator*(const double& rhs)
{
  vect4 ret;
  ret.a[0] = this->a[0] * rhs;
  ret.a[1] = this->a[1] * rhs;
  ret.a[2] = this->a[2] * rhs;
  ret.a[3] = this->a[3] * rhs;
  return ret;
}

vect4 vect4::operator/(const double& rhs)
{
  vect4 ret;
  ret.a[0] = this->a[0] / rhs;
  ret.a[1] = this->a[1] / rhs;
  ret.a[2] = this->a[2] / rhs;
  ret.a[3] = this->a[3] / rhs;
  return ret;
}

mat4 mat4::operator+(const mat4& rhs)
{
  mat4 ret;
  int I;
  for(I=0;I<16;I++)
  {
    ret.a[I] = this->a[I]+rhs.a[I];
  }

  return ret;
}
mat4 mat4::operator-(const mat4& rhs)
{
  mat4 ret;
  int I;
  for(I=0;I<16;I++)
  {
    ret.a[I] = this->a[I]-rhs.a[I];
  }
  return ret;
}

mat4 mat4::operator*(const double& rhs)
{
  mat4 ret;
  int I;
  for(I=0;I<16;I++)
  {
    ret.a[I] = this->a[I]*rhs;
  }
  return ret;
}
mat4 mat4::operator/(const double& rhs)
{
  mat4 ret;
  int I;
  for(I=0;I<16;I++)
  {
    ret.a[I] = this->a[I]/rhs;
  }
  return ret;
}

mat4 mat4::operator*(const mat4& rhs)
{
  int I,J,K;
  mat4 ret;
  double z;
  for(I=0;I<4;I++)
  {
    for(J=0;J<4;J++)
    {
      z = 0.0;
      for(K=0;K<4;K++)
      {
        z = z + this->at(I,K)*rhs.at(K,J);
      }
      ret.at(I,J) = z;
    }
  }
  return ret;
}

vect4 mat4::operator*(const vect4& rhs)
{
  vect4 ret;
  int I,J;
  double z;
  for(I=0;I<4;I++)
  {
    z = 0.0;
    for(J=0;J<4;J++)
    {
      z = z + this->at(I,J)*rhs.a[J];
    }
    ret.a[I] = z;
  }
  return ret;
}

/////////////
//functions//
/////////////

vect4 dtov4(double a0,double a1,double a2,double a3)
{
  vect4 ret;
  ret.a[0] = a0;
  ret.a[1] = a1;
  ret.a[2] = a2;
  ret.a[3] = a3;
  return ret;
}
vect4 v3tov4(vect3 v)
{
  vect4 ret;
  ret.a[0] = v.x;
  ret.a[1] = v.y;
  ret.a[2] = v.z;
  ret.a[3] = 1.0;
  return ret;
} //sets w to 1
vect3 v4tov3(vect4 v)
{
  vect3 ret;
  ret.x = v.a[0];
  ret.y = v.a[1];
  ret.z = v.a[2];
  return ret;
} //ignores w

mat4 m3tom4(mat3 q)
{
  int I,J;
  mat4 ret;
  for(I=0;I<4;I++)
  {
    for(J=0;J<4;J++)
    {
      if(I<3&&J<3)
      {
        ret.at(I,J) = q.a[I][J];
      }
      else
      {
        ret.at(I,J) = 0.0;
      }
    }
  }
  ret.at(3,3) = 1.0;
  return ret;
}

double mdet(mat4 ma)
{
  double a00,a01,a02,a03;
  double a10,a11,a12,a13;
  double a20,a21,a22,a23;
  double a30,a31,a32,a33;

  a00 = ma.at(0,0);
  a01 = ma.at(0,1);
  a02 = ma.at(0,2);
  a03 = ma.at(0,3);
  a10 = ma.at(1,0);
  a11 = ma.at(1,1);
  a12 = ma.at(1,2);
  a13 = ma.at(1,3);
  a20 = ma.at(2,0);
  a21 = ma.at(2,1);
  a22 = ma.at(2,2);
  a23 = ma.at(2,3);
  a30 = ma.at(3,0);
  a31 = ma.at(3,1);
  a32 = ma.at(3,2);
  a33 = ma.at(3,3);

  double det = a03*a12*a21*a30 -
        a02*a13*a21*a30 -
        a03*a11*a22*a30 +
        a01*a13*a22*a30 +
        a02*a11*a23*a30 -
        a01*a12*a23*a30 -
        a03*a12*a20*a31 +
        a02*a13*a20*a31 +
        a03*a10*a22*a31 -
        a00*a13*a22*a31 -
        a02*a10*a23*a31 +
        a00*a12*a23*a31 +
        a03*a11*a20*a32 -
        a01*a13*a20*a32 -
        a03*a10*a21*a32 +
        a00*a13*a21*a32 +
        a01*a10*a23*a32 -
        a00*a11*a23*a32 -
        a02*a11*a20*a33 +
        a01*a12*a20*a33 +
        a02*a10*a21*a33 -
        a00*a12*a21*a33 -
        a01*a10*a22*a33 +
        a00*a11*a22*a33;
  return det;
}

mat4 minverse(mat4 ma)
{
  mat4 mb;

  double a00,a01,a02,a03;
  double a10,a11,a12,a13;
  double a20,a21,a22,a23;
  double a30,a31,a32,a33;

  double b00,b01,b02,b03;
  double b10,b11,b12,b13;
  double b20,b21,b22,b23;
  double b30,b31,b32,b33;

  double det = 0.0;

  a00 = ma.at(0,0);
  a01 = ma.at(0,1);
  a02 = ma.at(0,2);
  a03 = ma.at(0,3);
  a10 = ma.at(1,0);
  a11 = ma.at(1,1);
  a12 = ma.at(1,2);
  a13 = ma.at(1,3);
  a20 = ma.at(2,0);
  a21 = ma.at(2,1);
  a22 = ma.at(2,2);
  a23 = ma.at(2,3);
  a30 = ma.at(3,0);
  a31 = ma.at(3,1);
  a32 = ma.at(3,2);
  a33 = ma.at(3,3);

  det = a03*a12*a21*a30 -
        a02*a13*a21*a30 -
        a03*a11*a22*a30 +
        a01*a13*a22*a30 +
        a02*a11*a23*a30 -
        a01*a12*a23*a30 -
        a03*a12*a20*a31 +
        a02*a13*a20*a31 +
        a03*a10*a22*a31 -
        a00*a13*a22*a31 -
        a02*a10*a23*a31 +
        a00*a12*a23*a31 +
        a03*a11*a20*a32 -
        a01*a13*a20*a32 -
        a03*a10*a21*a32 +
        a00*a13*a21*a32 +
        a01*a10*a23*a32 -
        a00*a11*a23*a32 -
        a02*a11*a20*a33 +
        a01*a12*a20*a33 +
        a02*a10*a21*a33 -
        a00*a12*a21*a33 -
        a01*a10*a22*a33 +
        a00*a11*a22*a33;

  if(ams::dabs(det)>1.0E-15)
  {
    b00 = -a13*a22*a31 + a12*a23*a31 + a13*a21*a32 - a11*a23*a32 - a12*a21*a33 + a11*a22*a33;
    b01 =  a03*a22*a31 - a02*a23*a31 - a03*a21*a32 + a01*a23*a32 + a02*a21*a33 - a01*a22*a33;
    b02 = -a03*a12*a31 + a02*a13*a31 + a03*a11*a32 - a01*a13*a32 - a02*a11*a33 + a01*a12*a33;
    b03 =  a03*a12*a21 - a02*a13*a21 - a03*a11*a22 + a01*a13*a22 + a02*a11*a23 - a01*a12*a23;
    b10 =  a13*a22*a30 - a12*a23*a30 - a13*a20*a32 + a10*a23*a32 + a12*a20*a33 - a10*a22*a33;
    b11 = -a03*a22*a30 + a02*a23*a30 + a03*a20*a32 - a00*a23*a32 - a02*a20*a33 + a00*a22*a33;
    b12 =  a03*a12*a30 - a02*a13*a30 - a03*a10*a32 + a00*a13*a32 + a02*a10*a33 - a00*a12*a33;
    b13 = -a03*a12*a20 + a02*a13*a20 + a03*a10*a22 - a00*a13*a22 - a02*a10*a23 + a00*a12*a23;
    b20 = -a13*a21*a30 + a11*a23*a30 + a13*a20*a31 - a10*a23*a31 - a11*a20*a33 + a10*a21*a33;
    b21 =  a03*a21*a30 - a01*a23*a30 - a03*a20*a31 + a00*a23*a31 + a01*a20*a33 - a00*a21*a33;
    b22 = -a03*a11*a30 + a01*a13*a30 + a03*a10*a31 - a00*a13*a31 - a01*a10*a33 + a00*a11*a33;
    b23 =  a03*a11*a20 - a01*a13*a20 - a03*a10*a21 + a00*a13*a21 + a01*a10*a23 - a00*a11*a23;
    b30 =  a12*a21*a30 - a11*a22*a30 - a12*a20*a31 + a10*a22*a31 + a11*a20*a32 - a10*a21*a32;
    b31 = -a02*a21*a30 + a01*a22*a30 + a02*a20*a31 - a00*a22*a31 - a01*a20*a32 + a00*a21*a32;
    b32 =  a02*a11*a30 - a01*a12*a30 - a02*a10*a31 + a00*a12*a31 + a01*a10*a32 - a00*a11*a32;
    b33 = -a02*a11*a20 + a01*a12*a20 + a02*a10*a21 - a00*a12*a21 - a01*a10*a22 + a00*a11*a22;
    b00 = b00/det;
    b01 = b01/det;
    b02 = b02/det;
    b03 = b03/det;
    b10 = b10/det;
    b11 = b11/det;
    b12 = b12/det;
    b13 = b13/det;
    b20 = b20/det;
    b21 = b21/det;
    b22 = b22/det;
    b23 = b23/det;
    b30 = b30/det;
    b31 = b31/det;
    b32 = b32/det;
    b33 = b33/det;
    mb.at(0,0) = b00;
    mb.at(0,1) = b01;
    mb.at(0,2) = b02;
    mb.at(0,3) = b03;
    mb.at(1,0) = b10;
    mb.at(1,1) = b11;
    mb.at(1,2) = b12;
    mb.at(1,3) = b13;
    mb.at(2,0) = b20;
    mb.at(2,1) = b21;
    mb.at(2,2) = b22;
    mb.at(2,3) = b23;
    mb.at(3,0) = b30;
    mb.at(3,1) = b31;
    mb.at(3,2) = b32;
    mb.at(3,3) = b33;
  }
  //this was STUPID. Gaah. Computer algebra system saves the day? I'd be surprised if this didn't end up *slower* than gaussian elimination. Don't do this again!
  return mb;
}

};
