
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <amsutil/amsutil.hpp>

namespace ams
{

void identity_permutation(long N, ams::amsarray<long> *permutation)
{
  long I;
  permutation->resize(N);
  for(I=0;I<N;I++)
  {
    permutation->at(I) = N;
  }
}

void invert_permutation(ams::amsarray<long> *permutation)
{
  long I;
  ams::amsarray<long> perm2 = *permutation;
  for(I=0;I<perm2.size();I++)
  {
    if(perm2[I]>=0&&perm2[I]<permutation->size())
    {
      permutation->at(perm2[I]) = I;
    }
  }
  return;
}

void reverse_permutation(ams::amsarray<long> *permutation)
{
  long I;
  ams::amsarray<long> perm2 = *permutation;
  for(I=0;I<perm2.size();I++)
  {
    permutation->at(I) = perm2[perm2.size()-I-1];
  }
  return;
}


  sorttree::sorttree()
  {
    ind = -1;
    parent = NULL;
    leftchild = NULL;
    rightchild = NULL;
    return;
  }

  sorttree::~sorttree()
  {
    ind = -1;
    parent = NULL;
    if(leftchild!=NULL)
    {
      delete leftchild;
      leftchild = NULL;
    }
    if(rightchild!=NULL)
    {
      delete rightchild;
      rightchild = NULL;
    }
    return;
  }

  //iterates over the sorttree, starting from the root and proceeding until it returns NULL
  sorttree* sorttree_next(sorttree *current, int *enterfrom)
  {
    sorttree* ret;
    if(*enterfrom==0) //if coming from the parent node
    {
      if(current->leftchild!=NULL)
      {
        *enterfrom = 0;
        ret = current->leftchild;
      }
      else if(current->rightchild!=NULL)
      {
        *enterfrom = 0;
        ret = current->rightchild;
      }
      else
      {
        if(current == current->parent->leftchild)
        {
          *enterfrom = 1;
          ret = current->parent;
        }
        else if(current==current->parent->rightchild)
        {
          *enterfrom = 2;
          ret = current->parent;
        }
        else
        {
          printf("Warning! Child node not child of either of parent's nodes. Memory leak imminent!\n");
        }
      }
    }
    if(*enterfrom==1) //if coming from the left child node
    {
      if(current->rightchild!=NULL)
      {
        *enterfrom = 0;
        ret = current->rightchild;
      }
      else
      {
        if(current->parent!=NULL)
        {
          if(current == current->parent->leftchild)
          {
            *enterfrom = 1;
            ret = current->parent;
          }
          else if(current==current->parent->rightchild)
          {
            *enterfrom = 2;
            ret = current->parent;
          }
          else
          {
            printf("Warning! Child node not child of either of parent's nodes. Memory leak imminent!\n");
          }
        }
        else
        {
          //this is the root node - we are done
          *enterfrom = -1;
          ret = NULL;
        }
      }
    }

    if(*enterfrom==2) //if coming from the right child node
    {
      if(current->parent!=NULL)
      {
        if(current == current->parent->leftchild)
        {
          *enterfrom = 1;
          ret = current->parent;
        }
        else if(current==current->parent->rightchild)
        {
          *enterfrom = 2;
          ret = current->parent;
        }
        else
        {
          printf("Warning! Child node not child of either of parent's nodes. Memory leak imminent!\n");
        }
      }
      else
      {
        //this is the root node - we are done
        *enterfrom = -1;
        ret = NULL;
      }
    }

    return ret;
  }

  sorttree& sorttree::operator=(const sorttree& other)
  {
    if(this!=&other)
    {
        ind = other.ind;
        parent = other.parent;
        leftchild = other.leftchild;
        rightchild = other.rightchild;
    }
    return *this;
  }

  sorttree::sorttree(const sorttree& other)
  {
    ind = -1;
    parent = NULL;
    leftchild = NULL;
    rightchild = NULL;
    if(this!=&other)
    {
      *this = other;
    }
    return;
  }

  //generates a permutation list from a sort-tree
  void sorttree_collect(sorttree *rootnode, amsarray<long> *permutation)
  {
    permutation->resize(0);
    sorttree *nodeptr = rootnode;
    int enterfrom;

    enterfrom = 0;
    while(nodeptr!=NULL)
    {
      if(nodeptr->leftchild==NULL && nodeptr->rightchild==NULL)
      {
        //end node
        permutation->append(nodeptr->ind);
      }
      else if(nodeptr->leftchild==NULL&&enterfrom==0)
      { //one-way branch to the right
        permutation->append(nodeptr->ind);
      }
      else if(enterfrom==1)
      {
        //coming back from left-child
        permutation->append(nodeptr->ind);
      }
      nodeptr = sorttree_next(nodeptr,&enterfrom);
    }

    permutation->shrink_to_fit();
    return;
  }

  //generates a permutation list from a sort-tree
  void sorttree_collect(sorttree *rootnode, amsarray<long> *permutation, long N)
  {
    long K;
    permutation->resize(N);
    sorttree *nodeptr = rootnode;
    int enterfrom;

    enterfrom = 0;
    K = 0;
    while(nodeptr!=NULL)
    {
      if(nodeptr->leftchild==NULL && nodeptr->rightchild==NULL)
      {
        //end node
        permutation->at(K) = nodeptr->ind;
        K = K + 1;
      }
      else if(nodeptr->leftchild==NULL&&enterfrom==0)
      { //one-way branch to the right
        permutation->at(K) = nodeptr->ind;
        K = K + 1;
      }
      else if(enterfrom==1)
      {
        //coming back from left-child
        permutation->at(K) = nodeptr->ind;
        K = K + 1;
      }
      nodeptr = sorttree_next(nodeptr,&enterfrom);
    }

    //permutation->shrink_to_fit();
    return;
  }

};
